SystemOrganization addCategory: #WeatherStation!

Model subclass: #WeatherDisplay
	instanceVariableNames: 'mqttClient data sensors temperaturesPlot humiditiesPlot pressurePlot windspeedPlot rainfallPlot barometer thermometer hygrometer windCompass windSpeedometer rainguage'
	classVariableNames: 'Debug'
	poolDictionaries: ''
	category: 'WeatherStation'!

!WeatherDisplay class methodsFor: 'class initialization' stamp: 'tpr 3/2/2017 17:01'!
debug: aBoolean
	"self debug: true"
	"self debug: false"
	Debug := aBoolean! !

!WeatherDisplay class methodsFor: 'plotting' stamp: 'tpr 3/31/2017 18:23'!
gridMarkersFrom: startMinute to: endMinute along: xOrY do: oneArgBlock for: aPlotMorph 
	"Weather data is plotted by the minute derived from the actual reading dat&time. So we need to 
	build a set of positions that nicely reflect time and date at the scale implied by the difference 
	between start & end"
	
	| closestDesiredSpacing totalMinutes |
	closestDesiredSpacing := 25.
	totalMinutes := endMinute - startMinute -1.
	"Find the finest-grain we can without spacing closer than 25 pixels."
	(totalMinutes isZero or: [ (aPlotMorph drawBounds width / totalMinutes) > closestDesiredSpacing ])
		ifTrue:
			[ startMinute
				to: endMinute
				do: oneArgBlock ]
		ifFalse:
			[ | pixelsPerHour |
			pixelsPerHour := aPlotMorph drawBounds width / (totalMinutes / 60).
			pixelsPerHour > closestDesiredSpacing
				ifTrue:
					[ startMinute
						to: endMinute
						do:
							[ : minute | (minute rem: 60) = 0 ifTrue: [ oneArgBlock value: minute ] ] ]
				ifFalse:
					[ | pixelsPerQDay |
					pixelsPerQDay := aPlotMorph drawBounds width / (totalMinutes / 480).
					pixelsPerQDay > closestDesiredSpacing
						ifTrue:
							[ startMinute
								to: endMinute
								do:
									[ : minute | |dt|
									dt := (DateAndTime fromSeconds: minute * 60).
									(dt minute = 0 and: [(dt hour24 \\ 3) = 0]) ifTrue: [ oneArgBlock value: minute ] ] ]
						ifFalse:
							[ | pixelsPerDay |
					pixelsPerDay := aPlotMorph drawBounds width / (totalMinutes / 1440).
					pixelsPerDay > closestDesiredSpacing
						ifTrue:
							[ startMinute
								to: endMinute
								do:
									[ : minute | |dt|
									dt := (DateAndTime fromSeconds: minute * 60).
									(dt minute = 0 and: [dt hour24 = 0]) ifTrue: [ oneArgBlock value: minute ] ] ]
						ifFalse:
							[] ] ] ]! !

!WeatherDisplay class methodsFor: 'plotting' stamp: 'tpr 3/31/2017 18:25'!
labelFor: proposedString
	^proposedString! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 3/30/2017 16:24'!
addData: newDataPoint
	"store the new data"

	"debugging output"
	Debug ifTrue: [Transcript show: newDataPoint asString; cr].
	data add: newDataPoint! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 4/15/2017 10:33'!
addData: newDataPoint fromSensor: aSensor for: aPlotMorph
	"store the new data and update the plot/series if aPlotMorph is non-nil.
	Used in the mqtt subscription Blocks; we return the newly added corrected data to help with updating any live-value displays"

	| newData |
	self addData: newDataPoint.
	newData := newDataPoint correctFor: aSensor.
	aPlotMorph ifNotNil:[:p|
		p series: aSensor commonName addPoint: newData].
	^newData y! !

!WeatherDisplay methodsFor: 'initialize-release' stamp: 'tpr 3/2/2017 16:54'!
connectToBroker
	"set up the mqttClient to talk to my broker, then subscribe to relevant topics"
	
	mqttClient := MQTTClient new.
	mqttClient
		keepAliveTime: 300;
		connectTo: 'Amtiskaw' port: nil.
		
	self subscribeToStatus;
		subscribeToOutdoors;
		subscribeToIndoors

	! !

!WeatherDisplay methodsFor: 'time' stamp: 'tpr 3/9/2017 14:30'!
currentTime
	
	^DateAndTime now! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/31/2017 17:35'!
dataOfType: aType sensor: aSensorOrID betweenDates: date1 and: date2
	"return a list of all the raw collected data that matches aType and was recorded between the two dates"
	"look for the sensor by its name and then ID"
	| sensor |
	sensor := self sensorAtName: aSensorOrID.
	"if we don't find it, maybe aSensorOrID is the actual sensor"
	sensor ifNil:[(sensors includes: aSensorOrID) ifTrue:[sensor := aSensorOrID] ifFalse:[^nil]].
	
	^data select: [:item|
		item matchesType: aType sensor: sensor betweenDates: date1 and: date2]! !

!WeatherDisplay methodsFor: 'plotting' stamp: 'tpr 4/15/2017 10:12'!
examplePlotHumiditiesForHours: numHours

	|end start series|
	end := DateAndTime now.
	start := end - numHours hours.
	humiditiesPlot := TimeDataPlotMorph new.
	humiditiesPlot extent: 800@400;
		crosshairStyle: #off; "<- turn off the moving crosshair cursor"
		lensStyle: #off;
		xAxisFormatter: [:x |String streamContents: [:strm|  (DateAndTime fromSeconds: x * 60) printHMOn: strm]]; 
		xAxisGridUnit: WeatherDisplay; "<- this makes the label layout nice for Dates"
		yAxisFormatter: [:y| '%-0.1e' printf: y];
		color: Color veryLightGray twiceLighter;
		yLabelsOnLeft: false;
		limitMaxY: 100;
		limitMinY: 30;
		setMaxLines: 12;
		timeRangeOfPlot: numHours hours;
		title: 'example humidity weather data'.

	series := humiditiesPlot series: #garage.
	series color: Color red;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #garage betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #guestroom.
	series color: Color yellow;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #guestroom betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #office.
	series color: Color blue darker;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #office betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #outdoors.
	series color: Color darkGray;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #outdoors betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #'001AF983'.
	series color: Color black;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #'001AF983' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #'001AFD29'.
	series color: Color red ;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #'001AFD29' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #'001AF80B'.
	series color: Color green;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #'001AF80B' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #'001AFDBD'.
	series color: Color brown;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #'001AFDBD' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #'001AF86D'.
	series color: Color salmon;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #'001AF86D' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := humiditiesPlot series: #'00C5F1F4'.
	series color: Color blue lighter;
			type: #line;
			drawPoints: false;
			width: 1.
	self humidityDataForSensor: #'00C5F1F4' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	humiditiesPlot openInWindowLabeled: 'Humidity plotting'.
	humiditiesPlot startStepping.

	self setHygrometer: HygrometerDialMorph new openInWorld
	
				 ! !

!WeatherDisplay methodsFor: 'plotting' stamp: 'tpr 4/12/2017 13:23'!
examplePlotPressureForHours: numHours

	|end start series|
	end := DateAndTime now.
	start := end - numHours hours.
	pressurePlot := TimeDataPlotMorph new.
	pressurePlot extent: 800@400;
		crosshairStyle: #off; "<- turn off the moving crosshair cursor"
		lensStyle: #off;
		xAxisFormatter: [:x |String streamContents: [:strm|  (DateAndTime fromSeconds: x * 60) printHMOn: strm]]; 
		xAxisGridUnit: WeatherDisplay; "<- this makes the label layout nice for Dates"
		yAxisFormatter: [:y| '%-0.1e' printf: y];
		limitMaxY: 1070;
		limitMinY: 970;
		color: Color veryLightGray twiceLighter;
		yLabelsOnLeft: false;
		timeRangeOfPlot: numHours hours;
		title: 'example pressure weather data'.

	series := pressurePlot series: #outdoors.
	series color: Color darkGray;
			type: #line;
			drawPoints: false;
			width: 1.
	self pressureDataForSensor: #outdoors betweenDates: start  and: end do: [:pt| series addPoint: pt].


	pressurePlot openInWindowLabeled: 'Pressure plotting'.
	pressurePlot startStepping.
	
	self setBarometer: BarometerMorph new openInWorld
	
				 ! !

!WeatherDisplay methodsFor: 'plotting' stamp: 'tpr 4/7/2017 11:34'!
examplePlotRainfallForHours: numHours

	|end start series|
	end := DateAndTime now.
	start := end - numHours hours.
	rainfallPlot := TimeDataPlotMorph new.
	rainfallPlot extent: 800@400;
		crosshairStyle: #off; "<- turn off the moving crosshair cursor"
		lensStyle: #off;
		xAxisFormatter: [:x |String streamContents: [:strm|  (DateAndTime fromSeconds: x * 60) printHMOn: strm]]; 
		xAxisGridUnit: WeatherDisplay; "<- this makes the label layout nice for Dates"
		yAxisFormatter: [:y| '%-0.1e' printf: y];
		color: Color veryLightGray twiceLighter;
		yLabelsOnLeft: false;
		timeRangeOfPlot: numHours hours;
		title: 'example rainfall weather data'.

	series := rainfallPlot series: #outdoors.
	series color: Color darkGray;
			type: #line;
			drawPoints: false;
			width: 1.
	self rainfallDataForSensor: #outdoors betweenDates: start  and: end do: [:pt| series addPoint: pt].


	rainfallPlot openInWindowLabeled: 'Rainfall plotting'.
	rainfallPlot startStepping
	
				 ! !

!WeatherDisplay methodsFor: 'plotting' stamp: 'tpr 4/5/2017 11:16'!
examplePlotSensorOffsets

	|pm series start end |
	end := DateAndTime now.
	start := end - 12 hours.
	pm := PlotMorph new.
	pm extent: 800@400;
		crosshairStyle: #off; "<- turn off the moving crosshair cursor"
		lensStyle: #off;
		xAxisFormatter: [:x |String streamContents: [:strm|  (DateAndTime fromSeconds: x * 60) printHMOn: strm]]; 
		xAxisGridUnit: WeatherDisplay; "<- this makes the label layout nice for Dates"
		yAxisFormatter: [:y| '%-0.1e' printf: y];
		color: Color veryLightGray twiceLighter;
		yLabelsOnLeft: false;
		title: 'example weather data'.

	series := pm series: #'001AF983'.
	series color: Color blue darker;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #'001AF983' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := pm series: #'00C3F109'.
	series color: Color salmon;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #'00C3F109' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	pm changed.
	pm openInWindowLabeled: 'Sensor offset display'
	
				 ! !

!WeatherDisplay methodsFor: 'plotting' stamp: 'tpr 4/15/2017 10:14'!
examplePlotTempsForHours: numHours

	|end start series|
	end := DateAndTime now.
	start := end - numHours hours.
	temperaturesPlot := TimeDataPlotMorph new.
	temperaturesPlot extent: 800@400;
		crosshairStyle: #off; "<- turn off the moving crosshair cursor"
		lensStyle: #off;
		xAxisFormatter: [:x |String streamContents: [:strm|  (DateAndTime fromSeconds: x * 60) printHMOn: strm]]; 
		xAxisGridUnit: WeatherDisplay; "<- this makes the label layout nice for Dates"
		yAxisFormatter: [:y| '%-0.1e' printf: y];
		color: Color veryLightGray twiceLighter;
		yLabelsOnLeft: false;
		limitMaxY: 30;
		limitMinY: 0;
		setMaxLines: 12;
		timeRangeOfPlot: numHours hours;
		title: 'example temperature weather data'.

	series := temperaturesPlot series: #garage.
	series color: Color red;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #garage betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #guestroom.
	series color: Color yellow;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #guestroom betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #office.
	series color: Color blue darker;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #office betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #outdoors.
	series color: Color darkGray;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #outdoors betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #'001AF983'.
	series color: Color black;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #'001AF983' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #'001AFD29'.
	series color: Color red ;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #'001AFD29' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #'001AF80B'.
	series color: Color green;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #'001AF80B' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #'001AFDBD'.
	series color: Color brown;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #'001AFDBD' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #'001AF86D'.
	series color: Color salmon;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #'001AF86D' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	series := temperaturesPlot series: #'00C5F1F4'.
	series color: Color blue lighter;
			type: #line;
			drawPoints: false;
			width: 1.
	self temperatureDataForSensor: #'00C5F1F4' betweenDates: start  and: end do: [:pt| series addPoint: pt].

	temperaturesPlot openInWindowLabeled: 'Example-2 weather data plotting'.
	temperaturesPlot startStepping.
	
	self setThermometer: ThermometerDialMorph new openInWorld
	
				 ! !

!WeatherDisplay methodsFor: 'plotting' stamp: 'tpr 4/15/2017 10:15'!
examplePlotWindSpeedForHours: numHours

	|end start series|
	end := DateAndTime now.
	start := end - numHours hours.
	windspeedPlot := TimeDataPlotMorph new.
	windspeedPlot extent: 800@400;
		crosshairStyle: #off; "<- turn off the moving crosshair cursor"
		lensStyle: #off;
		xAxisFormatter: [:x |String streamContents: [:strm|  (DateAndTime fromSeconds: x * 60) printHMOn: strm]]; 
		xAxisGridUnit: WeatherDisplay; "<- this makes the label layout nice for Dates"
		yAxisFormatter: [:y| '%-0.1e' printf: y];
		color: Color veryLightGray twiceLighter;
		yLabelsOnLeft: false;
		timeRangeOfPlot: numHours hours;
		title: 'example windspeed weather data'.

	series := windspeedPlot series: #outdoors.
	series color: Color darkGray;
			type: #line;
			drawPoints: false;
			width: 1.
	self windSpeedDataForSensor: #outdoors betweenDates: start  and: end do: [:pt| series addPoint: pt].


	windspeedPlot openInWindowLabeled: 'Windspeed plotting'.
	windspeedPlot startStepping.
	
	self setWindCompass: CompassDialMorph new openInWorld
	
				 ! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/30/2017 18:16'!
humidityDataForSensor: aSensorOrID betweenDates: date1 and: date2
	"return a list of all the collected humidity data for aSensorOrID  recorded between the two dates"
	|oc|
	oc := OrderedCollection new: 50. "wild guess at size needed"
	self humidityDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: [:d| oc add: d].
	^oc! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/31/2017 17:32'!
humidityDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: aBlock
	"process a list of all the collected humidity data for aSensorOrID recorded between the two dates via aBlock"
	|minutes1 minutes2 sensor |
	"look for the sensor by its name and then ID"
	sensor := self sensorAtName: aSensorOrID.
	"if we don't find it, maybe aSensorOrID is the actual sensor"
	sensor ifNil:[(sensors includes: aSensorOrID) ifTrue:[sensor := aSensorOrID] ifFalse:[^self]].
	minutes1 := date1 asMinutes.
	minutes2 := date2 asMinutes.	
	data do: [:item|
		(item isHumidityDataForSensor: sensor betweenMinutes: minutes1 and: minutes2) ifTrue: [ aBlock value: (item correctFor: sensor) ]]! !

!WeatherDisplay methodsFor: 'initialize-release' stamp: 'tpr 3/28/2017 12:02'!
initialize

	data := OrderedCollection new: 1000.
	sensors := Dictionary new:10.! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/30/2017 18:16'!
pressureDataForSensor: aSensorOrID betweenDates: date1 and: date2
	"return a list of all the collected pressure data for aSensorOrID  recorded between the two dates"
	|oc|
	oc := OrderedCollection new: 50. "wild guess at size needed"
	self pressureDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: [:d| oc add: d].
	^oc! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/31/2017 17:33'!
pressureDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: aBlock
	"process a list of all the collected pressure data for aSensorOrID recorded between the two dates via aBlock"
	|minutes1 minutes2 sensor |
	"look for the sensor by its name and then ID"
	sensor := self sensorAtName: aSensorOrID.
	"if we don't find it, maybe aSensorOrID is the actual sensor"
	sensor ifNil:[(sensors includes: aSensorOrID) ifTrue:[sensor := aSensorOrID] ifFalse:[^self]].
	minutes1 := date1 asMinutes.
	minutes2 := date2 asMinutes.	
	data do: [:item|
		(item isPressureDataForSensor: sensor betweenMinutes: minutes1 and: minutes2) ifTrue: [ aBlock value: (item correctFor: sensor) ]]! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/30/2017 18:16'!
rainfallDataForSensor: aSensorOrID betweenDates: date1 and: date2
	"return a list of all the collected rainfall data for aSensorOrID  recorded between the two dates"
	|oc|
	oc := OrderedCollection new: 50. "wild guess at size needed"
	self rainfallDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: [:d| oc add: d].
	^oc! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/31/2017 17:33'!
rainfallDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: aBlock
	"process a list of all the collected rainfall data for aSensorOrID recorded between the two dates via aBlock"
	|minutes1 minutes2 sensor |
	"look for the sensor by its name and then ID"
	sensor := self sensorAtName: aSensorOrID.
	"if we don't find it, maybe aSensorOrID is the actual sensor"
	sensor ifNil:[(sensors includes: aSensorOrID) ifTrue:[sensor := aSensorOrID] ifFalse:[^self]].
	minutes1 := date1 asMinutes.
	minutes2 := date2 asMinutes.	
	data do: [:item|
		(item isRainfallDataForSensor: sensor betweenMinutes: minutes1 and: minutes2) ifTrue: [ aBlock value: (item correctFor: sensor) ]]! !

!WeatherDisplay methodsFor: 'sensor details' stamp: 'tpr 3/28/2017 12:14'!
registerSensorID: anID
	"add the sensor to our list in basic form. Details about it can be added later"

	^sensors at: anID put: (ExternalSensor new sensorID: anID name: nil)! !

!WeatherDisplay methodsFor: 'sensor details' stamp: 'tpr 3/30/2017 16:28'!
sensorAtID: aSensorID
	"return the ExternalSensor known by aSensorID; if this is a new ID make a blank sensor for it.
	Note the two important differences to #sensorAtName:
		- we only look at the raw name of the sensor
		- we create a new one in case of unknown sensors"

	^sensors at: aSensorID ifAbsent: [self registerSensorID: aSensorID]! !

!WeatherDisplay methodsFor: 'sensor details' stamp: 'tpr 3/30/2017 16:29'!
sensorAtName: aSensorNameOrID
	"return the ExternalSensor known by aSensorNameOrID
		Note the two important differences to #sensorAtID:
		- we look at both the raw  nad common name of the sensor
		- we do not create a new one in case of unknown sensors"

	^sensors detect: [:s| s isKnownAs: aSensorNameOrID] ifNone: [nil]! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 3/28/2017 12:30'!
sensorFrom: topicString andData: dataString do: aBlock
	"extract a sensor id and data from the topic and data string, and pass them to aBlock"
	
	| sensorID |
	sensorID := (topicString findTokens: '/') last asSymbol.
	aBlock value: (self sensorAtID: sensorID) value:  dataString asString utf8ToSqueak ! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 3/28/2017 12:30'!
sensorFrom: topicString andNumericData: dataString do: aBlock
	"extract a sensor id and numeric data from the topic and data string, and pass them to aBlock"
	
	| sensorID tempNum |
	sensorID := (topicString findTokens: '/') last asSymbol.
	tempNum := Number readFrom: dataString asString utf8ToSqueak ifFail:[^nil].
	aBlock value: (self sensorAtID: sensorID) value: tempNum! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 4/7/2017 21:35'!
setBarometer: aRotaryDialMorph
	"temporary hack"
	barometer := aRotaryDialMorph! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 4/15/2017 10:12'!
setHygrometer: aRotaryDialMorph
	"temporary hack"
	hygrometer := aRotaryDialMorph! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 4/15/2017 10:14'!
setThermometer: aRotaryDialMorph
	"temporary hack"
	thermometer := aRotaryDialMorph! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 4/15/2017 10:11'!
setWindCompass: aRotaryDialMorph
	"temporary hack"
	windCompass := aRotaryDialMorph! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 4/15/2017 10:10'!
setWindSpeedomometer: aRotaryDialMorph
	"temporary hack"
	windSpeedometer := aRotaryDialMorph! !

!WeatherDisplay methodsFor: 'initialize-release' stamp: 'tpr 3/31/2017 18:17'!
subscribeToIndoors.
	
	mqttClient onTopic: 'weatherstation/boardtemp' do: [:t :m|
		self sensorFrom: #garage andNumericData: m do:[:sensor : d|
			self addData: (SensorTemperatureDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: temperaturesPlot]].
	
	mqttClient onTopic: 'weatherstation/indoortemp/#' do: [:t :m|
		self sensorFrom: t andNumericData: m do:[:sensor : d|
			self addData: (SensorTemperatureDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: temperaturesPlot]].
	
	mqttClient onTopic: 'weatherstation/indoorhumidity/#' do: [:t :m|
		self sensorFrom: t andNumericData: m do:[:sensor : d|
			self addData: (SensorHumidityDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: humiditiesPlot]].
	! !

!WeatherDisplay methodsFor: 'initialize-release' stamp: 'tpr 4/15/2017 10:08'!
subscribeToOutdoors
	
	| newData |
	mqttClient onTopic: 'weatherstation/pressure' do: [:t :m|
		self sensorFrom: #weatherpi andNumericData: m do:[:sensor : d|
			newData := self addData: (SensorPressureDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: pressurePlot.
		barometer ifNotNil: [:bm| bm inputValue: newData]]].
	
	mqttClient onTopic: 'weatherstation/outdoortemp' do: [:t :m|
		self sensorFrom: #weatherpi andNumericData: m do:[:sensor : d|
			newData := self addData: (SensorTemperatureDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: temperaturesPlot.
		thermometer ifNotNil: [:bm| bm inputValue: newData]]].
	
	mqttClient onTopic: 'weatherstation/outdoorhumidity/#' do: [:t :m|
		self sensorFrom: #weatherpi andNumericData: m do:[:sensor : d|
			newData := self addData: (SensorHumidityDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: humiditiesPlot.
		hygrometer ifNotNil: [:bm| bm inputValue: newData]]].
	
	mqttClient onTopic: 'weatherstation/raintotal' do: [:t :m|
		self sensorFrom: #weatherpi andNumericData: m do:[:sensor : d|
			newData := self addData: (SensorRainfallDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: rainfallPlot.
		rainguage ifNotNil: [:bm| bm inputValue: newData]]].

	mqttClient onTopic: 'weatherstation/windspeed' do: [:t :m|
		self sensorFrom: #weatherpi andNumericData: m do:[:sensor : d|
			newData := self addData: (SensorWindSpeedDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: windspeedPlot.
		windSpeedometer ifNotNil: [:bm| bm inputValue: newData]]].
	
	mqttClient onTopic: 'weatherstation/winddirection' do: [:t :m|
		self sensorFrom: #weatherpi andNumericData: m do:[:sensor : d|
			newData := self addData: (SensorWindDirectionDataPoint sensor: sensor  time: self currentTime data: d) fromSensor: sensor for: nil.
		windCompass ifNotNil: [:bm| bm inputValue: newData]]]
	! !

!WeatherDisplay methodsFor: 'initialize-release' stamp: 'tpr 3/31/2017 17:47'!
subscribeToStatus
	"we need to maintain some status data about the equipment"
	
	mqttClient onTopic: 'weatherstation/status/#' do: [:t :m|
		self sensorFrom: t andData: m do:[:sensor : d|
			self updateSensor: sensor statusTo:(d truncateTo: 50)]]! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/30/2017 18:15'!
temperatureDataForSensor: aSensorOrID betweenDates: date1 and: date2
	"return a list of all the collected temperature data for aSensorOrID  recorded between the two dates"
	|oc|
	oc := OrderedCollection new: 50. "wild guess at size needed"
	self temperatureDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: [:d| oc add: d].
	^oc! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/31/2017 17:33'!
temperatureDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: aBlock
	"process a list of all the collected temperature data for aSensorOrID recorded between the two dates via aBlock"
	|minutes1 minutes2 sensor |
	"look for the sensor by its name and then ID"
	sensor := self sensorAtName: aSensorOrID.
	"if we don't find it, maybe aSensorOrID is the actual sensor"
	sensor ifNil:[(sensors includes: aSensorOrID) ifTrue:[sensor := aSensorOrID] ifFalse:[^self]].
	minutes1 := date1 asMinutes.
	minutes2 := date2 asMinutes.	
	data do: [:item|
		(item isTemperatureDataForSensor: sensor betweenMinutes: minutes1 and: minutes2) ifTrue: [ aBlock value: (item correctFor: sensor) ]]! !

!WeatherDisplay methodsFor: 'recording data' stamp: 'tpr 3/31/2017 17:49'!
updateSensor: aSensorID statusTo: aString
	"track sensor status messages without keeping a long list of them"
	Transcript show: aSensorID rawName printString , ' status: ', aString; cr
	! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/30/2017 18:16'!
windDirectionDataForSensor: aSensorOrID betweenDates: date1 and: date2
	"return a list of all the collected windDirection data for aSensorOrID  recorded between the two dates"
	|oc|
	oc := OrderedCollection new: 50. "wild guess at size needed"
	self windDirectionDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: [:d| oc add: d].
	^oc! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/31/2017 17:33'!
windDirectionDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: aBlock
	"process a list of all the collected windDirection data for aSensorOrID recorded between the two dates via aBlock"
	|minutes1 minutes2 sensor |
	"look for the sensor by its name and then ID"
	sensor := self sensorAtName: aSensorOrID.
	"if we don't find it, maybe aSensorOrID is the actual sensor"
	sensor ifNil:[(sensors includes: aSensorOrID) ifTrue:[sensor := aSensorOrID] ifFalse:[^self]].
	minutes1 := date1 asMinutes.
	minutes2 := date2 asMinutes.	
	data do: [:item|
		(item isWindDirectionDataForSensor: sensor betweenMinutes: minutes1 and: minutes2) ifTrue: [ aBlock value: (item correctFor: sensor) ]]! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/30/2017 18:16'!
windSpeedDataForSensor: aSensorOrID betweenDates: date1 and: date2
	"return a list of all the collected windSpeed data for aSensorOrID  recorded between the two dates"
	|oc|
	oc := OrderedCollection new: 50. "wild guess at size needed"
	self windSpeedDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: [:d| oc add: d].
	^oc! !

!WeatherDisplay methodsFor: 'specific searches' stamp: 'tpr 3/31/2017 17:34'!
windSpeedDataForSensor: aSensorOrID betweenDates: date1 and: date2 do: aBlock
	"process a list of all the collected windSpeed data for aSensorOrID recorded between the two dates via aBlock"
	|minutes1 minutes2 sensor |
	"look for the sensor by its name and then ID"
	sensor := self sensorAtName: aSensorOrID.
	"if we don't find it, maybe aSensorOrID is the actual sensor"
	sensor ifNil:[(sensors includes: aSensorOrID) ifTrue:[sensor := aSensorOrID] ifFalse:[^self]].
	minutes1 := date1 asMinutes.
	minutes2 := date2 asMinutes.	
	data do: [:item|
		(item isWindSpeedDataForSensor: sensor betweenMinutes: minutes1 and: minutes2) ifTrue: [ aBlock value: (item correctFor: sensor) ]]! !

!Duration methodsFor: '*WeatherStation-time' stamp: 'tpr 3/3/2017 17:26'!
asMinutes


	^ seconds quo: SecondsInMinute! !

!DateAndTime methodsFor: '*WeatherStation-time' stamp: 'tpr 3/3/2017 17:26'!
asMinutes
	"Return the number of minutes since the Squeak epoch"
	^ (self - (self class epoch offset: offset)) asMinutes! !

!DateAndTime methodsFor: '*WeatherStation-time' stamp: 'tpr 3/5/2017 11:44'!
printHMOn: aStream
	"Print just hh:mm"
	aStream
		nextPutAll: (self hour asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (self minute asString padded: #left to: 2 with: $0)! !

Object subclass: #ExternalSensor
	instanceVariableNames: 'sensorID sensorName offsets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WeatherStation'!

!ExternalSensor methodsFor: 'accessing' stamp: 'tpr 3/27/2017 19:38'!
commonName
	"if I have a set name return it (for graph plotting etc) otherwise retrun my sensorID"
	^sensorName ifNil: [sensorID asString]! !

!ExternalSensor methodsFor: 'calibration handling' stamp: 'tpr 3/31/2017 17:11'!
correctData: rawData type: dataType
	"correct the rawData if appropriate, or return the rawData"

	^ rawData + (offsets ifNil:[0] ifNotNil:[ offsets at: dataType ifAbsent:[0]])! !

!ExternalSensor methodsFor: 'accessing' stamp: 'tpr 3/30/2017 16:03'!
isKnownAs: aSensorName
	"am I know by this name? Test my sensorID and sensorName"
	^aSensorName = sensorID or: [aSensorName = sensorName]! !

!ExternalSensor methodsFor: 'printing' stamp: 'tpr 3/29/2017 11:40'!
printOn: aStream

	super printOn: aStream.
	aStream nextPut: $(.
	self commonName printOn: aStream.
	aStream nextPut: $); space! !

!ExternalSensor methodsFor: 'accessing' stamp: 'tpr 3/30/2017 13:46'!
rawName
	"return my sensorID"
	^sensorID! !

!ExternalSensor methodsFor: 'initialize-release' stamp: 'tpr 3/27/2017 19:29'!
sensorID: aSensorID name: aSensorName
	"I must have a sensorID - the name used by mqtt to report my data, for example. I should have a common name, typically the location or whatever that I report upon."
	sensorID := aSensorID.
	sensorName := aSensorName.! !

!ExternalSensor methodsFor: 'calibration handling' stamp: 'tpr 3/31/2017 17:04'!
setOffset: type to: val
	"set the offset needed for data of type"

	offsets ifNil:[ offsets := Dictionary new:2].
	^offsets at: type put: val! !

Object subclass: #SensorDataPoint
	instanceVariableNames: 'sensorID timeInMinutes data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WeatherStation'!

!SensorDataPoint commentStamp: 'tpr 3/23/2017 19:09' prior: 0!
SensorDataPoint stores the fairly raw data from a sensor used in the WeatherStation project. Subclasses specialise what kind of data is held, along with providing a way to specify if the data is for temperature, humidity, pressure, smelliness or sexual desirability.

sensorID - typically a Symbol letting us know which sensor provided the data

timeInMinutes - it's more compact to store, and faster to compare date/time marks when converted to a minutes value, which conveniently fits in a SmallInteger for the plausible future.

data - data. the stuff we're interested in reading. Often a number, obviously.!

!SensorDataPoint class methodsFor: 'json-reading' stamp: 'tpr 3/30/2017 15:16'!
constructFromJson: json
	"build an instance from the JSon data - an Array with the data type, the sensor raw name, time in minutes and the actual data"
	|types|
	types := Dictionary new:10.
	types
		at: 'temp' put: SensorTemperatureDataPoint;
		at: 'humidity' put: SensorHumidityDataPoint;
		at: 'pressure' put: SensorPressureDataPoint;
		at: 'rainfall' put: SensorRainfallDataPoint;
		at: 'winddirection' put: SensorWindDirectionDataPoint;
		at: 'windspeed' put: SensorWindSpeedDataPoint.
	^(types at: (json at: 1)) new constructFromJson: json! !

!SensorDataPoint class methodsFor: 'instance creation' stamp: 'tpr 3/28/2017 12:33'!
sensor: aSensor time: aDateTime data: someStuff

	^super new sensor: aSensor time: aDateTime data: someStuff
! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 19:27'!
betweenDates: datetime1 and: datetime2
	"say whether my time fits the query; this is a naive user-centred method for quick testing. Use #betweenMinutes:and: for faster code"

	^self dateAndTime between: datetime1 and: datetime2! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 14:47'!
betweenMinutes: datetimeMinutes1 and: datetimeMinutes2
	"say whether my time fits the query. Remember this requires time in minutes for both arguments"

	^timeInMinutes between: datetimeMinutes1 and: datetimeMinutes2! !

!SensorDataPoint methodsFor: 'json-reading' stamp: 'tpr 3/30/2017 15:13'!
constructFromJson: json
	"build an instance from the JSon data - an Array with the data type, the sensor raw name, time in minutes and the actual data"
	sensorID := (json at: 2) asSymbol.
	timeInMinutes := json at:3.
	data := json at: 4! !

!SensorDataPoint methodsFor: 'calibration handling' stamp: 'tpr 3/31/2017 17:01'!
correctFor: aSensor
	"Ask the sensor to use its calibration to correct my data"
	^timeInMinutes @ (aSensor correctData: data type: self type)! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 19:27'!
dateAndTime

	^timeInMinutes ifNil:[ nil] ifNotNil:[:tm| DateAndTime fromSeconds: tm *60]! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 19:40'!
isHumidityDataForSensor: aSensorID betweenMinutes: minute1 and: minute2
	"am I humidity data for the named sensor between the times specified?
	Nope; see subclasses as well"
	^false! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 19:41'!
isPressureDataForSensor: aSensorID betweenMinutes: minute1 and: minute2
	"am I pressure data for the named sensor between the times specified?
	Nope; see subclasses as well"
	^false! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 19:41'!
isRainfallDataForSensor: aSensorID betweenMinutes: minute1 and: minute2
	"am I rainfall data for the named sensor between the times specified?
	Nope; see subclasses as well"
	^false! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 19:40'!
isTemperatureDataForSensor: aSensorID betweenMinutes: minute1 and: minute2
	"am I temperature data for the named sensor between the times specified?
	Nope; see subclasses as well"
	^false! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 19:41'!
isWindDirectionDataForSensor: aSensorID betweenMinutes: minute1 and: minute2
	"am I wind direction data for the named sensor between the times specified?
	Nope; see subclasses as well"
	^false! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/24/2017 19:41'!
isWindSpeedDataForSensor: aSensorID betweenMinutes: minute1 and: minute2
	"am I wind speed data for the named sensor between the times specified?
	Nope; see subclasses as well"
	^false! !

!SensorDataPoint methodsFor: 'json-writing' stamp: 'tpr 3/30/2017 21:50'!
jsonWriteOn: aStream

	aStream nextPutAll: '@', SensorDataPoint.
	(Array with: self type with: sensorID  with: timeInMinutes with: data) jsonWriteOn: aStream! !

!SensorDataPoint methodsFor: 'testing' stamp: 'tpr 3/30/2017 16:33'!
matchesType: aTypePatternString sensor: aSensor betweenDates: datetime1 and: datetime2
	"The most general and slowest test to find out if my type (which is generally discovered by checking my class) matches against the aTypePatternString argument *and* the sensor *and* was tagged between the two dateTimes.
	There are many faster approaches for specific needs"

	^(aTypePatternString match: self type) and: [aSensor isKnownAs: sensorID] and: [self betweenDates: datetime1 and: datetime2]! !

!SensorDataPoint methodsFor: 'printing' stamp: 'tpr 3/24/2017 19:29'!
printOn: aStream

	super printOn: aStream.
	aStream space nextPutAll: 'sensor: '.
	sensorID printOn: aStream.
	aStream space; nextPutAll: 'time: '.
	self dateAndTime printOn: aStream.
	aStream space; nextPutAll: 'data: '.
	data printOn: aStream
	! !

!SensorDataPoint methodsFor: 'initialize-release' stamp: 'tpr 3/31/2017 17:48'!
sensor: aSensor time: aDateTime data: someStuff
	"fill with your lovely data"

	"data can be anything"
	data := someStuff.
	sensorID := aSensor rawName.	
	"set my timeInMinutes to reflect aDateTime"
	timeInMinutes := aDateTime asMinutes! !

SensorDataPoint subclass: #SensorHumidityDataPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WeatherStation'!

!SensorHumidityDataPoint methodsFor: 'comparing' stamp: 'tpr 3/30/2017 18:36'!
isHumidityDataForSensor: aSensor betweenMinutes: minute1 and: minute2
	"am I humidity data for the named sensor between the times specified?
	We know I am humidity data already, so check the sensor id and times"

	^(aSensor isKnownAs: sensorID) and: [self betweenMinutes: minute1 and: minute2]! !

!SensorHumidityDataPoint methodsFor: 'comparing' stamp: 'tpr 3/24/2017 16:12'!
type

	^#humidity! !

SensorDataPoint subclass: #SensorPressureDataPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WeatherStation'!

!SensorPressureDataPoint methodsFor: 'calibration handling' stamp: 'tpr 3/31/2017 17:21'!
correctFor: aSensor
	"Ask the sensor to use its calibration to correct my data and then re-scale it from KPa to mBar"
	^1@10 * (super correctFor: aSensor)! !

!SensorPressureDataPoint methodsFor: 'comparing' stamp: 'tpr 3/30/2017 18:37'!
isPressureDataForSensor: aSensor betweenMinutes: minute1 and: minute2
	"am I pressure data for the named sensor between the times specified?
	We know I am pressure data already, so check the sensor id and times"

	^(aSensor isKnownAs: sensorID) and: [self betweenMinutes: minute1 and: minute2]! !

!SensorPressureDataPoint methodsFor: 'comparing' stamp: 'tpr 3/24/2017 16:13'!
type

	^#pressure! !

SensorDataPoint subclass: #SensorRainfallDataPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WeatherStation'!

!SensorRainfallDataPoint methodsFor: 'comparing' stamp: 'tpr 3/30/2017 18:37'!
isRainfallDataForSensor: aSensor betweenMinutes: minute1 and: minute2
	"am I rainfall data for the named sensor between the times specified?
	We know I am rainfall data already, so check the sensor id and times"

	^(aSensor isKnownAs: sensorID) and: [self betweenMinutes: minute1 and: minute2]! !

!SensorRainfallDataPoint methodsFor: 'comparing' stamp: 'tpr 3/24/2017 16:37'!
type

	^#rainfall! !

SensorDataPoint subclass: #SensorTemperatureDataPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WeatherStation'!

!SensorTemperatureDataPoint methodsFor: 'comparing' stamp: 'tpr 3/30/2017 16:17'!
isTemperatureDataForSensor: aSensor betweenMinutes: minute1 and: minute2
	"am I temperature data for the sensor between the times specified?
	We know I am temperature data already, so check the sensor id and times"

	^(aSensor isKnownAs: sensorID) and: [self betweenMinutes: minute1 and: minute2]! !

!SensorTemperatureDataPoint methodsFor: 'comparing' stamp: 'tpr 3/24/2017 16:08'!
type
	^#temp! !

SensorDataPoint subclass: #SensorWindDirectionDataPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WeatherStation'!

!SensorWindDirectionDataPoint methodsFor: 'comparing' stamp: 'tpr 3/30/2017 18:37'!
isWindDirectionDataForSensor: aSensor betweenMinutes: minute1 and: minute2
	"am I wind direction data for the named sensor between the times specified?
	We know I am wind direction data already, so check the sensor id and times"

	^(aSensor isKnownAs: sensorID) and: [self betweenMinutes: minute1 and: minute2]! !

!SensorWindDirectionDataPoint methodsFor: 'comparing' stamp: 'tpr 3/24/2017 16:37'!
type

	^#winddirection! !

SensorDataPoint subclass: #SensorWindSpeedDataPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WeatherStation'!

!SensorWindSpeedDataPoint methodsFor: 'comparing' stamp: 'tpr 3/30/2017 18:37'!
isWindSpeedDataForSensor: aSensor betweenMinutes: minute1 and: minute2
	"am I wind speed data for the named sensor between the times specified?
	We know I am wind speed data already, so check the sensor id and times"

	^(aSensor isKnownAs: sensorID) and: [self betweenMinutes: minute1 and: minute2]! !

!SensorWindSpeedDataPoint methodsFor: 'comparing' stamp: 'tpr 3/24/2017 16:37'!
type

	^#windspeed! !
